"""PEAK - Python-based framework for heterogeneous agent communities.

PEAK is a framework for developing communities of heterogeneous agents.
This communities are multi-agent systems that can coexist and exchange
resources and information with each other easly.

isort: skip_file
"""

from peak.core import *
from peak.message import *
from peak.template import *
from peak.agents import *
from peak.behaviours import *
from peak.logging import *

__author__ = "GECAD"
__email__ = "brgri@isep.ipp.pt"
__version__ = "1.2.4"
